var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var LocalStorageController = require('../controller/LocalStorageController');
var EventDispatcher = require('../event/EventDispatcher');
var RequestEvent = require('../event/RequestEvent');
var LanguageEvent = require('../event/LanguageEvent');
var BaseRequest = require('../request/BaseRequest');
var LanguageConfigModel = require('./LanguageConfigModel');
/**
 * The LanguageModel...
 *
 * @class LanguageModel
 * @module StructureJS
 * @submodule model
 * @constructor
 * @author Robert S. (www.codeBelt.com)
 */
var LanguageModel = (function (_super) {
    __extends(LanguageModel, _super);
    function LanguageModel() {
        _super.call(this);
        this._request = null;
        this._availableLanguagesDictionary = [];
        this._localStorageController = null;
        this.currentLanguage = null;
        this.data = null;
        this._localStorageController = new LocalStorageController();
        this._localStorageController.setNamespace('StructureJS.');
        this.currentLanguage = this._localStorageController.getItem('language', true);
    }
    /**
     * The ...
     *
     * @method loadConfig
     * @param path {string} The path to the main language config json file.
     * @example
     {
     "data": [
         {
             "id": "en-US",
             "lang": "English",
             "text": "English",
             "path": "data/languages/main.en.json"
         },
         {
             "id": "es-ES",
             "lang": "Spanish",
             "text": "Español",
             "path": "data/languages/main.sp.json"
         }
     ]
 }
     */
    LanguageModel.prototype.loadConfig = function (path) {
        this._request = new BaseRequest(path);
        this._request.addEventListener(RequestEvent.SUCCESS, this.onConfigLoaded, this);
        this._request.load();
    };
    /**
     *
     * @method setLang
     * @param value
     */
    LanguageModel.prototype.setLang = function (value) {
        this.currentLanguage = value;
    };
    /**
     *
     * @method loadLanguageData
     * @param path {LanguageConfigModel}
     * @protected
     */
    LanguageModel.prototype.loadLanguageData = function (vo) {
        this._localStorageController.addItem('language', vo.id, true);
        this._request = new BaseRequest(vo.path);
        this._request.addEventListener(RequestEvent.SUCCESS, this.onLanguageDataLoad, this);
        this._request.load();
    };
    /**
     *
     * @method getSupportedLanguages
     */
    LanguageModel.prototype.getSupportedLanguages = function () {
        var temp = [];
        for (var key in this._availableLanguagesDictionary) {
            temp.push(this._availableLanguagesDictionary[key]);
        }
        return temp;
    };
    /**
     *
     * @method loadLanguageById
     */
    LanguageModel.prototype.loadLanguageById = function (id) {
        var vo = this.getLangConfigById(id);
        this.loadLanguageData(vo);
    };
    /**
     *
     * @method getLangConfigById
     * @param id {string}
     * @return {LanguageConfigModel}
     * @protected
     */
    LanguageModel.prototype.getLangConfigById = function (id) {
        return this._availableLanguagesDictionary[id];
    };
    /**
     *
     * @method onConfigLoaded
     * @param event {RequestEvent}
     * @protected
     */
    LanguageModel.prototype.onConfigLoaded = function (event) {
        this._request.removeEventListener(RequestEvent.SUCCESS, this.onConfigLoaded, this);
        var firstLanguageId = null;
        var jsonData = JSON.parse(event.target.data);
        var vo;
        var len = jsonData.data.length;
        for (var i = 0; i < len; i++) {
            vo = new LanguageConfigModel(jsonData.data[i]);
            this._availableLanguagesDictionary[vo.id] = vo;
            // Save a reference to the first vo id so we can set that as the default language.
            if (firstLanguageId == null) {
                firstLanguageId = vo.id;
            }
        }
        // Checks if the language id that exists in LocalStorage is found in the _availableLanguagesDictionary.
        var languageIdFound = this.hasLanguage(this.currentLanguage);
        // If there is no default language set in LocalStorage then use the first one in the _availableLanguagesDictionary.
        this.currentLanguage = (languageIdFound) ? this.currentLanguage : firstLanguageId;
        this.dispatchEvent(new RequestEvent(LanguageEvent.CONFIG_LOADED, false, false, this.data));
        // Get the language vo and get the json file path to load that specific language.
        var currentLanguageModel = this.getLangConfigById(this.currentLanguage);
        this.loadLanguageData(currentLanguageModel);
    };
    /**
     *
     * @method onLanguageDataLoad
     * @param event {RequestEvent}
     * @protected
     */
    LanguageModel.prototype.onLanguageDataLoad = function (event) {
        this.data = JSON.parse(event.target.data);
        this._request.removeEventListener(RequestEvent.SUCCESS, this.onConfigLoaded, this);
        this._request = null;
        this.dispatchEvent(new RequestEvent(LanguageEvent.LANGUAGE_LOADED, false, false, this.data));
    };
    /**
     * We need to check if the language id that exists in LocalStorage matches an id in the _availableLanguagesDictionary.
     * If the application was updated and the language id's changed it would break the application.
     *
     * @method hasLanguage
     * @protected
     */
    LanguageModel.prototype.hasLanguage = function (languageId) {
        return !!this._availableLanguagesDictionary[languageId];
    };
    return LanguageModel;
})(EventDispatcher);
module.exports = LanguageModel;
//# sourceMappingURL=LanguageModel.js.map